/*
 * Copyright (c) 2015 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/* global SpeechSynthesisUtterance, speechSynthesis */

(function() {
    var textArray = [],
        currentTextIndex = 0,
        stopButtonFlag = 0,
        nextButtonFlag = 0,
        textDiv = null,
        speech = null,
        LANGUAGE = "en-US";

    /**
     * Read in the text data from an external file and display the text on the screen
     * @private
     */
    function loadText() {
        var textLocation = 'text/text.txt',
            charTemp = "",
            textTemp = "",
            textTempIndex = 0,
            textTotal = "",
            data = null,
            i = 0;

        textDiv = document.getElementById('textContent');

        // If it succeeds to access the location of the widget package, it retrieves the text file and displays the text data on the screen.
        function onResolveSuccess(dir) {
            try {
                data = dir.resolve(textLocation);
            } catch (e) {
                // If it fails to retrieve the text file, it displays an error message.
                textDiv.innerHTML = "<p style='color: #ff0000;'>Text File Error: " + e.message + "</p>";
            }

            if (data) {
                // The whole text is passed as an argument.
                data.readAsText(function(arg) {
                    textArray = [];
                    textTempIndex = 0;
                    // Remove any escape characters from the text data.
                    arg = arg.replace(/(\r\n|\n|\r)/gm, "");

                    for (i = 0; i < arg.length; i++) {
                        // It will read in each character of the text to determine if it is the end of a sentence.
                        charTemp = arg.charAt(i);
                        textTemp += charTemp;

                        // Whenever it encounters '.', '!' or '?' character, it is indicated to be the end of a sentence.
                        // A <p> tag will be created for each sentence.
                        if (charTemp === '.' || charTemp === '!' || charTemp === '?') {
                            textArray[textTempIndex] = textTemp;
                            textTotal += "<p id = line" + textTempIndex + ">" + textTemp + "</p>";
                            textTemp = "";
                            textTempIndex++;
                        }
                    }

                    // Insert the series of <p> tags into the html code so that the text gets displayed.
                    textDiv.innerHTML = textTotal;
                });
            }
        }

        // If it fails to access the location of the widget package, it will display an error message on the console.
        function onResolveError(e) {
            console.error("Error" + e.message);
        }

        // Resolves the location of the widget package using the tizen.filesystem.resolve() call.
        tizen.filesystem.resolve('wgt-package', onResolveSuccess, onResolveError, 'r');
    }

    /**
     * Change the icon of a button according to its staus
     * @private
     * @param {Boolean} isActive - the status of the icon
     * @param {String} buttonName - the name of the button
     */
    function changeIcon(isActive, buttonName) {
        var buttonId = buttonName + "-button",
            iconSrc = "";

        if (isActive === 1) {
            // When a button is active, the button should be displayed with a dark grey icon.
            iconSrc = buttonName + ".png";
        } else if (isActive === 0) {
            // When a button is inactive, the button should be displayed with a light grey icon.
            iconSrc = buttonName + "-inactive.png";
        }

        document.getElementById(buttonId).style.background = "url('/images/" + iconSrc + "') no-repeat center center";
    }

    /**
     * Enable specific buttons according to where the sentence beign read is placed
     * @private
     * @param {Number} textIndex - the index of the sentence being read
     */
    function enableButtons(textIndex) {
        if (textIndex <= 0) {
            // When it is reading the first sentence, only the next button is enabled.
            changeIcon(0, 'prev');
            changeIcon(1, 'next');
        } else if (textIndex > 0 && textIndex < textArray.length - 1) {
            // When it is reading any sentence in the middle, both next and prev buttons are enabled.
            changeIcon(1, 'prev');
            changeIcon(1, 'next');
        } else if (textIndex >= textArray.length - 1) {
            // When it is reading the last sentence, only the prev button is enabled.
            changeIcon(1, 'prev');
            changeIcon(0, 'next');
        }
    }

    /**
     * Scroll to a specific point to show the sentence beign read
     * @private
     * @param {Number} textIndex - the index of the sentence being read
     */
    function scrollTo(textIndex) {
        var currentPosition = 0,
            textDivPosition = 0,
            textDivOffHeight = textDiv.offsetTop;

        if (textIndex === 0) {
            // Always scroll to the top when the first sentence is being read.
            document.getElementById("main").scrollTop = 0;
        } else {
            // Otherwise, calculate the position of the sentence and move to the point.
            textDivPosition = document.getElementById('line' + textIndex).offsetTop;
            currentPosition = parseInt(textDivPosition - textDivOffHeight - 50, 10);
            document.getElementById("main").scrollTop = currentPosition;
        }
    }

    /**
     * Read the text using the Web Speech API.
     * @private
     * @param {Number} textIndex - the index of the sentence being read
     */
    function readText(textIndex) {
        // The SpeechSynthesisUtterance interface represents a speech request.
        // The interface contains the content the speech service should read and information about how to read it.
        speech = new SpeechSynthesisUtterance(textArray[textIndex]);
        speech.lang = LANGUAGE;

        // Set timeout for the speak function call
        // because calling it right after declaring message object does not work for multiple calls.
        setTimeout(function() {
            // The speak() method of the SpeechSynthesis interface adds an utterance to the utterance queue.
            speechSynthesis.speak(speech);
        }, 150);

        enableButtons(textIndex);
        scrollTo(textIndex);
        // Change the color of the text being read to darker grey.
        document.getElementById('line' + textIndex).style.color = "#666666";

        // The onend property represents an event handler that will run when the utterance has finished being spoken.
        speech.onend = function() {
            // When it finishes reading a sentence, change the color of the text back to lighter grey.
            document.getElementById('line' + textIndex).style.color = "#999999";
            // Check if the stop button has not been clicked.
            if (stopButtonFlag === 0) {
                if (textIndex === textArray.length - 1 && nextButtonFlag === 0) {
                    // After reading the last sentence, click the stop button automatically.
                    document.getElementById('stop-button').click();
                } else if (nextButtonFlag === 1) {
                    // If the next button has been clicked, change the nextButtonFlag back to 0.
                    nextButtonFlag = 0;
                } else {
                    // Otherwise, continue reading the next sentence.
                    readText(++currentTextIndex);
                }
            }
        };
    }

    /**
     * Add event listeners for buttons
     * @private
     */
    function buttonEvents() {
        var playBtn = document.getElementById('play-button'),
            pauseBtn = document.getElementById('pause-button'),
            resumeBtn = document.getElementById('resume-button'),
            prevBtn = document.getElementById('prev-button'),
            nextBtn = document.getElementById('next-button'),
            stopBtn = document.getElementById('stop-button');

        // Add event listener for play button.
        playBtn.addEventListener('click', function() {
            stopButtonFlag = 0;
            readText(currentTextIndex);
            playBtn.classList.add('hidden');
            pauseBtn.classList.remove('hidden');
            changeIcon(1, 'stop');
        });

        // Add event listener for pause button.
        pauseBtn.addEventListener('click', function() {
            // The pause() method of the SpeechSynthesis interface puts the SpeechSynthesis object into a paused state.
            speechSynthesis.pause();
            pauseBtn.classList.add('hidden');
            resumeBtn.classList.remove('hidden');
        });

        // Add event listener for resume button.
        resumeBtn.addEventListener('click', function() {
            // The resume() method of the SpeechSynthesis interface puts the SpeechSynthesis object into a non-paused state.
            // It resumes reading if it was already paused.
            speechSynthesis.resume();
            pauseBtn.classList.remove('hidden');
            resumeBtn.classList.add('hidden');
        });

        // Add event listener for prev button.
        prevBtn.addEventListener('click', function() {
            if (stopButtonFlag === 0 && currentTextIndex > 0) {
                nextButtonFlag = 1;
                currentTextIndex -= 1;
                // The cancel() method of the SpeechSynthesis interface removes all utterances from the utterance queue.
                speechSynthesis.cancel();
                readText(currentTextIndex);
            } else {
                console.log("There is nothing to play.");
            }
        });

        // Add event listener for next button.
        nextBtn.addEventListener('click', function() {
            if (stopButtonFlag === 0 && currentTextIndex < textArray.length - 1) {
                nextButtonFlag = 1;
                currentTextIndex++;
                speechSynthesis.cancel();
                readText(currentTextIndex);
            } else {
                console.log("There is nothing to play.");
            }
        });

        // Add event listener for stop button.
        stopBtn.addEventListener('click', function() {
            if (stopButtonFlag === 0) {
                // Reset everything to initial state.
                stopButtonFlag = 1;
                currentTextIndex = 0;
                speechSynthesis.cancel();
                scrollTo(0);

                playBtn.classList.remove('hidden');
                pauseBtn.classList.add('hidden');
                resumeBtn.classList.add('hidden');
                changeIcon(0, 'stop');
                changeIcon(0, 'prev');
                changeIcon(0, 'next');
            } else {
                console.log("there is nothing playing");
            }
        });

        // Add event listener for hardware back key
        window.addEventListener('tizenhwkey', function(e) {
            if (e.keyName === 'back') {
                tizen.application.getCurrentApplication().exit();
            }
        });
    }

    /**
     * Initiates the application
     * @private
     */
    function appInit() {
        loadText();
        buttonEvents();
    }

    // Call appInit function when the window is loaded.
    window.onload = function() {
        appInit();
    };
}());
